*/////////////// Save as any.prg /////////////////////////////////////////
*// The demonstration is realized as class CSqlCmdToXML and shows:
*// How to receive an outcome of the request to external SQL Server
*// and transform it into the XML-file by using command 
*// ADODB.Recordset.Save(MyXMLfile, adPersistXML)
*// If outcome is obtained, on completion it will be transformed
*// into VFP-cursor by using command XMLTOCURSOR()
*// (for VFP ver 7.0+(SP1VFP7.0) + MS SQL Server 7.0(SP2))
*//======================================================================
*// Specification using constants:
*//    RESULTCUR  - Name VFP-cursor as result
*//    RESULTXML  - Name XML-file as result
*//    STRCONNECT - ConnectionString to the MS SQL Server database Pubs
*//    SQLCMD     - SQL SELECT
*//======================================================================
*// Mailto:Drozdov@ics.perm.su
*// My Page:  http://vfpdmur.narod.ru/
*// ICS Page: http://ics.perm.ru/ [on Russian]
*// Date:     04/14/2001
*//======================================================================
*////////////////////////////////////////////////////////////////////////
*//
#DEFINE RESULTCUR	"curXMLresult"
#DEFINE RESULTXML	LOWER(ADDBS(SYS(2023))+SUBSTR(SYS(2015),3))+'.xml'
#DEFINE	STRCONNECT	"Provider=MSDATASHAPE;Data Provider=SQLOLEDB;";
	+"DSN=LocalServer;Description=Pubs Database;";
	+"DATABASE=pubs;Trusted_Connection=Yes;";
	+"APP=Microsoft(R) Visual FoxPro;"
#DEFINE SQLCMD	"SELECT au_id, au_lname, au_fname FROM dbo.authors"

*!*	#DEFINE SQLCMD	"SHAPE {SELECT au_id, au_lname, au_fname FROM dbo.authors} "; 
*!*		+ "APPEND ({SELECT TA.au_id, TS.title FROM dbo.titleauthor TA, dbo.titles TS "; 
*!*		+ "WHERE TA.title_id = TS.title_id} ";
*!*		+ "AS title_chap RELATE au_id TO au_id)"
*
* Hm... XMLTOCURSOR() can not transform SHAPE-result :-(
*
*///////////////////////////////////////////
*// Other constants
*//
#DEFINE	ERRORCAPTION	"Test SQL to XML"
#DEFINE CRLF			CHR(13) + CHR(10)
#DEFINE adStateOpen		1
#DEFINE adPersistXML	1

#DEFINE NERRINVALIDARG	11
#DEFINE NERRNAMENOTOBJ	1924

*///////////////////////////////////////////
*// Use class CSqlCmdToXML
*//
LOCAL lbResult as Boolean  
LOCAL lcFileOut as String 
LOCAL loSqlCmdToXML as CSqlCmdToXML && OF d:\myapp\vfp7\testdataodbc\sqltoxml.prg
loSqlCmdToXML = NEWOBJECT('CSqlCmdToXML')
IF VARTYPE(loSqlCmdToXML) # 'O'
	ACTIVATE SCREEN 	
	??CHR(7)
   	= MESSAGEBOX('Can not create CSqlCmdToXML object', 16, ERRORCAPTION)
	RETURN lbResult
ENDIF
lcFileOut = RESULTXML
WITH loSqlCmdToXML
	IF .Connect(STRCONNECT) ;
			AND .SQLExec(SQLCMD) ;
			AND .SaveAsXML(lcFileOut)
		WAIT WINDOW "The file: '"+PROPER(JUSTFNAME(lcFileOut))+"' was created!" NOWAIT
		=XMLTOCURSOR(lcFileOut, RESULTCUR, 512)
		IF USED(RESULTCUR)
			SELECT (RESULTCUR)
			BROWSE NORM
			lbResult = .T.
		ENDIF	
	ENDIF
ENDWITH		
RETURN lbResult

*////////////////////////////////
*// CSqlCmdToXML
*//
DEFINE CLASS CSqlCmdToXML AS Session  
	PROTECTED oCn As ADODB.Connection  
	PROTECTED oRS As ADODB.Recordset 

	FUNCTION CloseRecordset
	 	IF VARTYPE(This.oRs) = 'O'
		 	IF This.oRs.State = adStateOpen
		 		This.oRs.Close()   
			ENDIF
		ENDIF	
	ENDFUNC	

	FUNCTION DisConnect
		This.CloseRecordset()
	 	IF VARTYPE(This.oCn) = 'O'
		 	IF This.oCn.State = adStateOpen
		 		This.oCn.Close()   
			ENDIF
		ENDIF	
	ENDFUNC	

	PROCEDURE Destroy
		WITH This
			.DisConnect()
			.oRS = NULL
			.oCn = NULL
		ENDWITH	
	ENDPROC	

	FUNCTION Connect
	 	LPARAMETERS tcCntString
	 	IF VARTYPE(tcCntString) # 'C' OR EMPTY(tcCntString)
	 		ERROR NERRINVALIDARG, "tcCntString" 
	 		RETURN .F.
	 	ENDIF
	 	WITH This
	 		.DisConnect() 
		 	IF VARTYPE(.oCn) # 'O'
		 		.oCn = NEWOBJECT("ADODB.Connection")
		 	ENDIF	
		 	IF VARTYPE(.oCn) # 'O'
		 		RETURN .F.
		 	ENDIF
		 	WITH .oCn 	
			 	IF .State <> adStateOpen
	   			 	.Open(tcCntString)
				 	IF .State <> adStateOpen   
						RETURN .F.
					ENDIF
				ENDIF
			ENDWITH	
		ENDWITH		
	ENDFUNC	

	FUNCTION SQLExec
	 	LPARAMETERS tcSQLcmd
	 	IF VARTYPE(tcSQLcmd) # 'C' OR EMPTY(tcSQLcmd)
	 		ERROR NERRINVALIDARG, "tcSQLcmd" 
	 		RETURN .F.
	 	ENDIF
	 	IF VARTYPE(This.oCn) # 'O'
	 		ERROR NERRNAMENOTOBJ, This.Name+".oCn"
	 		RETURN .F.
	 	ENDIF
	 	WITH This
		 	.CloseRecordset()
		 	IF VARTYPE(.oRs) # 'O'
		 		.oRs = NEWOBJECT("ADODB.Recordset")
		 	ENDIF	
		 	IF VARTYPE(.oRs) # 'O'
		 		RETURN .F.
		 	ENDIF
		 	WITH .oRs	
			 	IF .State <> adStateOpen   
					.ActiveConnection = This.oCn		 	
				 	.Open(tcSQLcmd)
				 	IF .State <> adStateOpen   
						RETURN .F.
					ENDIF
				ENDIF
			ENDWITH	
		ENDWITH			
	ENDFUNC	

	FUNCTION SaveAsXML
	 	LPARAMETERS tcXmlFile
	 	IF VARTYPE(tcXmlFile) # 'C' OR EMPTY(tcXmlFile)
	 		ERROR NERRINVALIDARG, "tcXmlFile"
	 		RETURN .F.
	 	ENDIF
	 	IF VARTYPE(This.oRs) # 'O'
	 		ERROR NERRNAMENOTOBJ, This.Name+".oRs"
	 		RETURN .F.
	 	ENDIF
	 	WITH This.oRs
		 	IF .State = adStateOpen   
			 	IF FILE(tcXmlFile)
			 		ERASE (tcXmlFile)
			 	ENDIF	
			 	.Save(tcXmlFile, adPersistXML)
				RETURN FILE(tcXmlFile)			
			ELSE 	
	 			ERROR NERRINVALIDARG, This.Name+".oRs.State <> adStateOpen"
	 			RETURN .F.
			ENDIF
		ENDWITH	
	 	RETURN .F.
	ENDFUNC	
	
	PROCEDURE Error
		LPARAMETERS nError, cMethod, nLine
		IF VARTYPE(nError) # 'N'
			nError = 0
		ENDIF
		IF VARTYPE(cMethod) # 'C'
			cMethod = ""
		ENDIF
		IF VARTYPE(nLine) # 'N'
			nLine = 0
		ENDIF
		LOCAL ARRAY laErr[1]
		LOCAL lcMsg as String 
		= AERROR(laErr)
		lcMsg = "Error: "+LTRIM(STR(nError))
		DO CASE
		CASE INLIST(nError, 1427, 1429)
			lcMsg = lcMsg+CRLF;
				+ "OLE error message: "+laErr[3]+CRLF;
				+ "Application name: "+laErr[4]
			IF (!ISNULL(laErr[5]) AND LENC(laErr[5]) > 1)
				lcMsg = lcMsg+CRLF;
					+ "Help file name: "+laErr[5]
			ENDIF		
		CASE nError = 1526
			lcMsg = lcMsg+CRLF;
				+ "ODBC error message: "+laErr[3]+CRLF;
				+ "ODBC SQL state: "+laErr[4]+CRLF;
				+ "ODBC error number: "+LTRIM(STR(laErr[5]))  
		OTHERWISE
			lcMsg = lcMsg+CRLF;
				+ "Message: "+laErr[2]
		ENDCASE		
		lcMsg = lcMsg + CRLF;
			+ "Class: "+PROPER(This.Name)+CRLF;
			+ "Method: "+PROPER(cMethod)+CRLF;
			+ "Line: "+LTRIM(STR(nLine))
		IF VARTYPE(This.oCn) = 'O' 
			IF This.oCn.Errors.Count > 0
				lcMsg = lcMsg+CRLF+"~~~~~~~~~~~~"
				lcMsg = lcMsg+CRLF+"ADODB error message:"
				LOCAL lnCnt as Integer
				FOR lnCnt = 0 To This.oCn.Errors.Count-1
					lcMsg = lcMsg+CRLF+"Count: "+LTRIM(STR(lnCnt))+" ~~~~~~~~~~~~"
					lcMsg = lcMsg+CRLF+"Description: "+This.oCn.Errors(lnCnt).Description
					lcMsg = lcMsg+CRLF+"NativeError: "+LTRIM(STR(This.oCn.Errors(lnCnt).NativeError))
				ENDFOR
			ENDIF	
		ENDIF	  
		ACTIVATE SCREEN 	
		??CHR(7)
	   	= MESSAGEBOX(lcMsg, 16, ERRORCAPTION)
	ENDPROC
ENDDEFINE
*/////////////// End of any.prg /////////////////////////////////////////
